/*
 * $Id: xmgr.c,v 1.7 2003/08/07 15:54:33 oliver Exp $
 * 
 *
 * please refer to http://www.fhi-berlin.mpg.de/grz/pub/xfarbe/xfarbe-2.6/html/index.html 
 * for details about xfarbe and the file format
 *
 * 
 */
static char *SRCID_xmgr_c = "$Id: xmgr.c,v 1.7 2003/08/07 15:54:33 oliver Exp $";

#include <string.h>
#include <ctype.h>
#include "sysstuff.h"
#include "string2.h"
#include "futil.h"
#include "statutil.h"
#include "copyrite.h"
#include "smalloc.h"
#include "xmgr.h"


/* minimum parameter file; colours are defined in xmgr.h */
bool xf_write_XFarbe (int ncolours) {
  FILE *XFarbe;
  int i;
  
  if (!(XFarbe=fopen(XFARBE,"w"))){
    msg("error: Cannot open file " XFARBE "for writing.\n");
    return FALSE;
  }
  if (ncolours > XFARBE_NCOLS) {
    msg("warning: The requested number of levels %d is not available. You will \n"
	"         only get %d instead.\n",ncolours,XFARBE_NCOLS);
    ncolours = XFARBE_NCOLS;
  }

  msg("Writing resource file './" XFARBE "' for xfarbe with %d colours.\n",ncolours);
  dmsg("Use it with one of\n"
      "bash$ export XAPPLRESDIR=.\n"
      "tcsh> setenv XAPPLRESDIR .\n");
  fpxfi("ncol",ncolours);
  fpxfi("mode",0);
  fpxf ("autolev","off");
  fpxf ("xyratio","on");
  fpxf ("legend", "on");
  fpxf ("legend_format","%4.2f");
  fpxf ("x-annotation", "on");   /* annotation on but increment 0 */  
  fpxf ("y-annotation", "on");   /* allows data probing in xfarbe */
  fpxfi("x-annotation-incr",0);  /* _with_ real x and y values    */
  fpxfi("y-annotation-incr",0);  /* describe in */
  /* http://www.fhi-berlin.mpg.de/grz/pub/xfarbe/xfarbe-2.6/html/index.html#Data%20Probing */ 
  for(i=0;i<ncolours;i++) 
    fprintf(XFarbe,"xfarbe.col%d:   #%.6X\n",i,xfarbe_colours[i]);

  fclose(XFarbe);
  return TRUE;
}

/* append int+1 levels (upto value max) to the file */
void xf_append_levels (FILE *fData, int ncols, real max) {
  int i;
  real delta;

  delta = max/(ncols-1);
  fprintf(fData,"%d\n",ncols-1);        /* number of levels */
  fprintf(fData, "%d  %f\n",0,delta/2); /* first level filters out noise */
  for(i=1;i<ncols-1;i++)                /* equally spaced */
    fprintf(fData, "%d  %f\n",i,i*delta);
  fprintf(fData, "%d\n", ncols-1);      /* everything else */
  return;
}

void xf_append_axes_annotation (FILE *fData, real x1, real x2, 
			     real y1, real y2) {
  fprintf(fData,"%f  %f\n",x1, x2);
  fprintf(fData,"%f  %f\n",y1, y2);
  return;
}


bool bXmGrace(void)
{
  /* return (getenv("XMGRACE") != NULL); */
  return (getenv("GRACE_HOME") != NULL);
}


FILE *xmgropen(char *fn,char *title,char *subtitle,
	       char *xaxis,char *yaxis)
{
  FILE *xmgr;
  time_t t;
  
  xmgr=(FILE *)ffopen(fn,"w");
  fprintf(xmgr,"# Grace project file");
  fprintf(xmgr,"# This file was created by %s\n",Program());
  fprintf(xmgr,"# which is part of G R O M A C S:\n");
  fprintf(xmgr,"# %s\n",bromacs());
  time(&t);
  fprintf(xmgr,"# All this happened at: %s",ctime(&t));
  fprintf(xmgr,"#\n");
  fprintf(xmgr,"@version 50102\n"); 
  fprintf(xmgr,"@with g0\n"); 
  fprintf(xmgr,"@    title \"%s\"\n",title);
  fprintf(xmgr,"@    subtitle \"%s\"\n",subtitle);
  fprintf(xmgr,"@    xaxis  label \"%s\"\n",xaxis);
  fprintf(xmgr,"@    yaxis  label \"%s\"\n",yaxis);
  fprintf(xmgr,"@target G0.S0\n"); 
  fprintf(xmgr,"@type nxy\n"); 
  
  return xmgr;
}


void xmgr_file(char *fn,char *opts)
{
  char   doit[STRLEN];

  if (bDoView()) {
    sprintf(doit,"%s -geometry +100+100 %s %s &",
	    bXmGrace() ? "xmgrace" : "xmgr", opts ? opts : "",fn);
    system(doit);
  }
}

void xmgr_view(FILE *out,real xmin,real ymin,real xmax,real ymax)
{
  fprintf(out,"@ view %g, %g, %g, %g\n",xmin,ymin,xmax,ymax);
}

void xmgr_world(FILE *out,real xmin,real ymin,real xmax,real ymax)
{
  fprintf(out,"@ world xmin %g\n"
	  "@ world ymin %g\n"
	  "@ world xmax %g\n"
	  "@ world ymax %g\n",xmin,ymin,xmax,ymax);
}

void xmgr_legend(FILE *out,int nsets,char *setname[])
{
  int i;
  
  xmgr_view(out,0.15,0.15,0.75,0.85);
  fprintf(out,"@ legend on\n");
  fprintf(out,"@ legend box on\n");
  fprintf(out,"@ legend loctype view\n");
  fprintf(out,"@ legend %g, %g\n",0.78,0.8);
  fprintf(out,"@ legend length %d\n",2);
  for(i=0; (i<nsets); i++)
    if (bXmGrace())
      fprintf(out,"@ s%d legend \"%s\"\n",i,setname[i]);
    else
      fprintf(out,"@ legend string %d \"%s\"\n",i,setname[i]);
}

void xmgr_line_props(FILE *out, int NrSet, int LineStyle, int LineColor)
{
  fprintf(out, "@    with g0\n");
  fprintf(out, "@    s%d linestyle %d\n", NrSet, LineStyle);
  fprintf(out, "@    s%d color %d\n", NrSet, LineColor);
}

static char *LocTypeStr[] = { "view", "world" };
static char *BoxFillStr[] = { "none", "color", "pattern" };
 
void xmgr_box(FILE *out,
	      int LocType,
	      real xmin,real ymin,real xmax,real ymax,
	      int LineStyle,int LineWidth,int LineColor,
	      int BoxFill,int BoxColor,int BoxPattern)
{
  fprintf(out,"@with box\n");
  fprintf(out,"@    box on\n");
  fprintf(out,"@    box loctype %s\n",LocTypeStr[LocType]);
  fprintf(out,"@    box %g, %g, %g, %g\n",xmin,ymin,xmax,ymax);
  fprintf(out,"@    box linestyle %d\n",LineStyle);
  fprintf(out,"@    box linewidth %d\n",LineWidth);
  fprintf(out,"@    box color %d\n",LineColor);
  fprintf(out,"@    box fill %s\n",BoxFillStr[BoxFill]);
  fprintf(out,"@    box fill color %d\n",BoxColor);
  fprintf(out,"@    box fill pattern %d\n",BoxPattern);
  fprintf(out,"@box def\n");
}




static char *fgets3(FILE *fp)
{
  static char *ptr = NULL;
  static int  len  = STRLEN;
  char *p;
  int  slen;
  
  if (ptr==NULL)
    snew(ptr,len);

  if (fgets(ptr,len-1,fp) == NULL)
    return NULL;
  p = ptr;
  while ((strchr(ptr,'\n') == NULL) && (!feof(fp))) {
    /* This line is longer than len characters, let's increase len! */
    len += STRLEN;
    p   += STRLEN;
    srenew(ptr,len);
    if (fgets(p-1,STRLEN,fp) == NULL)
      break;
  }
  slen = strlen(ptr);
  if (ptr[slen-1] == '\n')
    ptr[slen-1] = '\0';
  return ptr;
}

static int wordcount(char *ptr)
{
  int i,n,is[2];
  int cur=0;
#define prev (1-cur)
  
  if (strlen(ptr) == 0)
    return 0;
  /* fprintf(stderr,"ptr='%s'\n",ptr); */
  n=1;
  for(i=0; (ptr[i] != '\0'); i++) {
    is[cur] = isspace(ptr[i]);
    if ((i > 0)  && (is[cur] && !is[prev]))
      n++;
    cur=prev;
  }
  return n;
}

int read_agr(char *fn,real ***y,int *ny)
{
  FILE   *fp;
  char   *ptr;
  char   *base=NULL;
  char   *fmt=NULL;
  int    k,line=0,nny,nx,maxx,rval;
  double lf;
  real   **yy=NULL;
  
  *ny  = 0;
  nny  = 0;
  nx   = 0;
  maxx = 0;
  fp   = ffopen(fn,"r");
  while ((ptr = fgets3(fp)) != NULL) {
    line++;
    trim(ptr);
    if ((ptr[0] != '@') && (ptr[0] != '#')) {
      if (nny == 0) {
	(*ny) = nny = wordcount(ptr);
	/* fprintf(stderr,"There are %d columns in your file\n",nny);*/
	if (nny == 0)
	  return 0;
	snew(yy,nny);
	snew(fmt,3*nny+1);
	snew(base,3*nny+1);
      }
      /* Allocate column space */
      if (nx >= maxx) {
	maxx+=1024;
	for(k=0; (k<nny); k++)
	  srenew(yy[k],maxx);
      }
      /* Initiate format string */
      fmt[0]  = '\0';
      base[0] = '\0';
      
      /* fprintf(stderr,"ptr='%s'\n",ptr);*/
      for(k=0; (k<nny); k++) {
	strcpy(fmt,base);
	strcat(fmt,"%lf");
	rval = sscanf(ptr,fmt,&lf);
	/* fprintf(stderr,"rval = %d\n",rval);*/
	if ((rval == EOF) || (rval == 0))
	  break;
	yy[k][nx] = lf;
	srenew(fmt,3*(nny+1)+1);
	srenew(base,3*nny+1);
	strcat(base,"%*s");
      }
      if (k != nny) {
	fprintf(stderr,"Only %d columns on line %d in file %s\n",
		k,line,fn);
	for( ; (k<nny); k++)
	  yy[k][nx] = 0.0;
      }
      nx++;
    }
  }
  ffclose(fp);
  
  *y = yy;
  
  return nx;
}

void dump_agr(char *fn,char *title,int nx,int ny,real **y)
{
  FILE *fp;
  int  i,j;
  
  fp=xmgropen(fn,title,"","X","Y");
  for(i=0; (i<nx); i++) {
    for(j=0; (j<ny); j++) {
      fprintf(fp,"  %12.5e",y[j][i]);
    }
    fprintf(fp,"\n");
  }
  fclose(fp);
}

