/*
 * $Id: xmgr.h,v 1.6 2002/08/20 23:09:46 oliver Exp $
 * 
 *       This source code is part of
 * 
 *        G   R   O   M   A   C   S
 * 
 * GROningen MAchine for Chemical Simulations
 * 
 *               VERSION 2.0
 * 
 * Copyright (c) 1991-1999
 * BIOSON Research Institute, Dept. of Biophysical Chemistry
 * University of Groningen, The Netherlands
 * 
 * Please refer to:
 * GROMACS: A message-passing parallel molecular dynamics implementation
 * H.J.C. Berendsen, D. van der Spoel and R. van Drunen
 * Comp. Phys. Comm. 91, 43-56 (1995)
 * 
 * Also check out our WWW page:
 * http://md.chem.rug.nl/~gmx
 * or e-mail to:
 * gromacs@chem.rug.nl
 * 
 * And Hey:
 * Green Red Orange Magenta Azure Cyan Skyblue
 */

#ifndef _xmgr_h
#define _xmgr_h

static char *SRCID_xmgr_h = "$Id: xmgr.h,v 1.6 2002/08/20 23:09:46 oliver Exp $";

#ifdef HAVE_IDENT
#ident	"@(#) xvgr.h 1.8 7/28/97"
#endif /* HAVE_IDENT */

#ifdef CPLUSPLUS
extern "C" {
#endif

#include "sysstuff.h"
#include "typedefs.h"
#include "macros.h"
#include "utilgmx.h"

/***************************************************
 *            xfarbe routines (2D density plots)
 ***************************************************/
extern bool xf_write_XFarbe (int);
/* write a parameter file with 16 colours from xfrabe_colours */
extern void xf_append_levels (FILE *, int, real);
/* append level definitions to a XFarbe data file 
   ncols - 1 levels equally spaced up to max,
   above max, all is coloured in the last colour, number ncols-1
 */
extern void xf_append_axes_annotation (FILE *, real x1, real x2, 
					 real y1, real y2);
/* label x axis from x1 to x2, &idem y axis */ 

/*
  write customised XFarbe file 
  use with 'export XAPPLRESDIR=.' which activates the level definitions at 
  the end of the data file

  extract colours with:
     gawk '{gsub("\#","0x"); printf ("%s, ", $2)}' cols
  from the xfarbe v2.5 default XFarbe file

  colours: 0xRRGGBB (2 bytes per red, green, blue)

  http://www.ndirect.co.uk/~thomas.green/javascripts/hexAndRGB.html
*/

static u_int xfarbe_colours[] = 
      {0x000099, 0x1111DD, 0x5555FF, 0x6677DD, 0x8899AA, 0xAABB66, 
       0xCCDD33, 0xEEEE00, 0xFFFF00, 0xFFDD00, 0xFFBB00, 0xFF9900,  
       0xFF6600, 0xEE1100, 0xBB0000, 0x770000};

/* it is not easy finding good colours---the xfarbe default is already
   pretty decent...  */
/*
  static u_int xfarbe_colours[] =
       {0x1400FF, 0x0010FF, 0x0066FF, 0x00A3FF, 0x00E0BB, 
       0x00F0AA, 0x33FF88, 0x00FF00, 0x14FF00, 0x52F000, 
       0x8FEE00, 0xCCDD22, 0xFFC000, 0xFFB800, 0xFF7A00,
       0xFF3D00, 0xFF0000, 0xFF007A, 0xFF00B8, 0xFF00F5 };
  */


#define XFARBE_NCOLS asize(xfarbe_colours)  
#define XFARBE "XFarbe"      /* name of the parameter file */
#define fpxf(resource,str) fprintf(XFarbe,"xfarbe.%s: %s\n",(resource),(str))
#define fpxfi(resource,i)  fprintf(XFarbe,"xfarbe.%s: %d\n",(resource),(i))


/***************************************************
 *            XGraph routines
 ***************************************************/
extern FILE *xgopen(char *fn,char *header,char *title,char *xaxis,char *yaxis);
/* Open a file, and write a title, and axis-labels in XGraph format */

extern void xgraph_file(char *fn,char *opts);
/* Starts xgraph with a file fn in the background,
 * opts (options to xgraph) may be NULL
 */

/***************************************************
 *            XMGR   DEFINITIONS
 ***************************************************/
enum {
  elNone, elSolid, elDotted, elDashed, 
  elLongDashed, elDotDashed, elNR
};
/* xmgr line-styles */

enum {  
  ecWhite, ecFrank, ecBlack=ecFrank,
  ecRed, ecGreen, ecBlue, ecYellow, ecBrown, ecGray, 
  ecPurple, ecLightBlue, ecViolet, ecHolland, ecLila, ecDarkGray, 
  ecAquamarine, ecOlive, ecNR
};
/* xmgr line-colors */

enum {
  eppNone, eppColor, eppPattern, eppNR
};
/* xmgr pattern type */

enum {
  evView, evWorld, evNR
};
/* view type */

/***************************************************
 *            XMGR   ROUTINES
 ***************************************************/

extern FILE *xmgropen(char *fn,char *title,char *subtitle,
                      char *xaxis,char *yaxis);
/* Open a file, and write a title, and axis-labels in Xmgr format */

extern void xmgr_file(char *fn,char *opts);
/* Starts xmgr with a file fn in the background,
 * opts (options to xmgr) may be NULL
 */

extern void xmgr_view(FILE *out,real xmin,real ymin,real xmax,real ymax);
/* Set the view in xmgr */

extern void xmgr_world(FILE *out,real xmin,real ymin,real xmax,real ymax);
/* Set the world in xmgr */

extern void xmgr_legend(FILE *out,int nsets,char *setname[]);
/* Make a legend box, and also modifies the view to make room for the legend */

extern void xmgr_line_props(FILE *out,int NrSet,int LineStyle,int LineColor);
/* Set xmgr line styles and colors */

extern void xmgr_box(FILE *out,
		     int LocType,
		     real xmin,real ymin,real xmax,real ymax,
		     int LineStyle,int LineWidth,int LineColor,
		     int BoxFill,int BoxColor,int BoxPattern);
/* Make a box */

extern int read_agr(char *fn,real ***y,int *ny);
/* Read an agr file for post processing. The number of rows is returned
 * fn is the filename, y is a pointer to a 2D array (to be allocated by
 * the routine) ny is the number of columns (including X if appropriate)
 */
 
extern void dump_agr(char *fn,char *title,int nx,int ny,real **y);
/* Quicly dump a two D array (y) of dimensions nx rows times
 * ny columns to a file.
 */


#ifdef CPLUSPLUS
}
#endif

#endif	/* _xmgr_h */
